#include-once
#include <WinAPI.au3>
#include <WinAPIFiles.au3>

#cs

	https://www.autoitscript.com/forum/topic/193277-ipc-udf-via-filemapping/

	Available functions:
	_FMIPC_Create
	_FMIPC_Create_ErrStr - for debug aid
	_FMIPC_Close
	_FMIPC_Read
	_FMIPC_Write
	_FMIPC_Write_ErrStr - for debug aid
	_FMIPC_SetName
	_FMIPC_SetSize
	_FMIPC_SetSize_ErrStr - for debug aid
	_FMIPC_SetTimeout

#ce



Global $g__FMIPC_Size = 1024 * 1024 * 2 ; 1 MB messages default max., change to your needs via _FMIPC_SetSize()
Global $g__FMIPC_Timeout = 500 ; ms., change to your needs via _FMIPC_SetTimeout()
Global $g__FMIPC_Version = '0.2019.06.10'
Global $g__FMIPC_h = 0, $g__FMIPC_p = 0, $g__FMIPC_t = 0, $g__FMIPC_n = "" ; do not touch this

; #FUNCTION# ;===============================================================================
;
; Name...........: _FMIPC_Create
; Description ...: Creates a FileMapping with the specified name.
; Syntax.........: _FMIPC_Create ([$sName])
; Parameters ....: $sName - The name for the FileMapping object
; Return values .: Success - Returns True
;                          - Sets @error to 0
;                  Failure - Returns False and sets @error:
;                  |1 - The FileMapping object was previously created
;                  |2 - Unable to create file mapping
; Remarks .......: If no $sName is given, the ScriptName + PID is used.
;                  in that way, one can do IPC to any running script by means of ProcessList()
; Author ........: argumentum
; Modified.......:
; Link ..........;
;
;==========================================================================================

Func _FMIPC_Create($sName = Default)
	If $sName = Default Then $sName = StringTrimRight(@ScriptName, 4) & String(@AutoItPID)
	If $g__FMIPC_h Then Return SetError(1, 0, "")
	Local $iErr = 0, $iExt = 0
	$g__FMIPC_h = _WinAPI_CreateFileMapping(-1, $g__FMIPC_Size, $sName)
	$iErr = @error
	$iExt = @extended
	If $iExt Or $iErr Or Not $g__FMIPC_h Then Return SetError(2, $iExt, "") ; Unable to create file mapping
	$g__FMIPC_p = _WinAPI_MapViewOfFile($g__FMIPC_h)
	$g__FMIPC_t = DllStructCreate('wchar[' & $g__FMIPC_Size / 2 & ']', $g__FMIPC_p) ; 16bit(2byte) UNICODE wide char
	;OnAutoItExitRegister("_FMIPC_Close") ; this makes a crash on exit !!!
	$g__FMIPC_n = $sName ; for _FMIPC_SetSize ()
	Return SetError(0, 0, $sName)
EndFunc   ;==>_FMIPC_Create

; #FUNCTION# ;===============================================================================
;
; Name...........: _FMIPC_Close
; Description ...: Closes the FileMapping object
; Syntax.........: _FMIPC_Close ()
; Parameters ....: none
; Return values .: none, work in proggress.  =/
; Author ........: argumentum
; Modified.......:
; Link ..........;
;
;==========================================================================================
Func _FMIPC_Close()
	Local $Err = 0, $Ext = 0
	If $g__FMIPC_h Then
		If Not _WinAPI_UnmapViewOfFile($g__FMIPC_p) Then $Err += 1
		If Not _WinAPI_CloseHandle($g__FMIPC_h) Then $Ext += 2
		$Err += $Ext
	EndIf
	If Not $Ext Then
		$g__FMIPC_p = 0
		$g__FMIPC_h = 0
		Return 0
	Else
		Return SetError($Err, $Ext, $Err + $Ext)
	EndIf
EndFunc   ;==>_FMIPC_Close

; #FUNCTION# ;===============================================================================
;
; Name...........: _FMIPC_Read
; Description ...: Reads values from the DllStruct.
; Syntax.........: _FMIPC_Read ()
; Parameters ....: none
; Return values .: Success - Returns read data
;                          - Sets @extended to number of read bytes
;                          - Sets @error to 0
; Author ........: argumentum
; Modified.......:
; Link ..........;
;
;==========================================================================================
Func _FMIPC_Read()
	If Not $g__FMIPC_h Then Return SetError(1, 0, "")
	Local $vData = DllStructGetData($g__FMIPC_t, 1)
	If $vData Then DllStructSetData($g__FMIPC_t, 1, "") ; this tells the _FMIPC_Write() that we can receive again
	Return SetError(@error, StringLen($vData), $vData)
EndFunc   ;==>_FMIPC_Read

; #FUNCTION# ;===============================================================================
;
; Name...........: _FMIPC_Write
; Description ...: Writes values to FileMapping object name
; Syntax.........: _FMIPC_Write ($sName , $vData, [$iNoWait])
; Parameters ....: $sName - FileMapping object name
;                  $vData - Data to write.
;                $iNoWait = 0 : (default) Wait for all flags/acknowledgements
;                $iNoWait = 1 : Don't wait for acknowledgement of reception
;                $iNoWait = 2 : Don't wait anything, blind write/overwrite ( not recomended ! )
;
; Return values .: Success - Returns the TimerInit()
;                          - Sets @error to 0
;                  Failure - Returns 0 and sets @error:
;                  |1 - _WinAPI_OpenFileMapping failed.
;                  |2 - _WinAPI_MapViewOfFile failed.
;                  |3 - did not confirm receiving.
;                  |4 - never sent, timed out waiting for no data in DllStruct
;                  |5 - $vData is larger than DllStruct declaration
;                  - Sets @extended to _WinAPI_GetLastError()
;
; Author ........: argumentum
; Note ..........: the OK flag is an empty string in the DllStruct
; Modified.......:
; Link ..........;
;
;==========================================================================================
Func _FMIPC_Write($sName, $vData, $iNoWait = 0)
	$iNoWait = Int($iNoWait)
	If StringLen($vData) * 2 > $g__FMIPC_Size Then Return SetError(5, 0, 0) ; $vData would not fit !!!
	Local $s = "", $Err = 0, $timer = TimerInit()
	Local $hMapping = _WinAPI_OpenFileMapping($sName)
	If Not $hMapping Then Return SetError(1, _WinAPI_GetLastError(), 0) ; _WinAPI_OpenFileMapping FAILED
	Local $pAddress = _WinAPI_MapViewOfFile($hMapping)
	If Not $pAddress Then Return SetError(2, _WinAPI_GetLastError(), 0) ; _WinAPI_MapViewOfFile FAILED
	Local $tData = DllStructCreate('wchar[' & $g__FMIPC_Size / 2 & ']', $pAddress) ; 16bit(2byte) UNICODE wide char
	Do
		If $iNoWait > 1 Then ExitLoop
		If TimerDiff($timer) > $g__FMIPC_Timeout Then ExitLoop ; this is to be adjusted to you expectetions
		$s = DllStructGetData($tData, 1)
	Until $s = ""
	If $s = "" Then
		DllStructSetData($tData, 1, $vData)
		_WinAPI_FlushViewOfFile($pAddress)
		$timer = TimerInit()
		Do
			If $iNoWait Then ExitLoop
			If TimerDiff($timer) > $g__FMIPC_Timeout Then ; this is to be adjusted to you expectetions
				DllStructSetData($tData, 1, "")
				$timer = 0
			EndIf
		Until DllStructGetData($tData, 1) = "" ; to know it was received, $timer will return its success
	EndIf
	_WinAPI_UnmapViewOfFile($pAddress)
	_WinAPI_CloseHandle($hMapping)
	If Not $timer Then $Err = 3 ; did not confirm receiving
	If $s Then
		$Err = 4 ; never sent
		$timer = 0 ; the TimerInit() is set to zero, can use it as "If Not _FMIPC_Write( ..."
	EndIf
	Return SetError($Err, 0, $timer)
EndFunc   ;==>_FMIPC_Write


; #FUNCTION# ;===============================================================================
;
; Name...........: _FMIPC_SetSize
; Description ...: Changes the size used in FileMapping
; Syntax.........: _FMIPC_SetSize ($iSize)
; Parameters ....: $iSize - Set the size of the FileMapping object in bytes
; Return values .: Success - Returns $iSize
;                          - Sets @error to 0
;                  Failure - Returns False and sets @error:
;                  |9 - changing the size "on the fly" not yet implemented
;                  @extended = 1 :
; Author ........: argumentum
; Modified.......: new in 0.2180.04.?? - Todo: continue and test
; Link ..........;
;
;==========================================================================================
Func _FMIPC_SetSize($iSize = Default)
	If $iSize = Default Then Return SetError(0, 2, $g__FMIPC_Size)
	$iSize = Int($iSize)
	If $iSize < 32 Then $iSize = 32
	Local $i, $j, $k
	If $g__FMIPC_h Then
		Return SetError(9, 0, $g__FMIPC_Size) ; it crashes !!!,  grrr.
		$i = _FMIPC_Close() ;  it fails !!!, ... retry?. Need testing.
		$j = @error
		$k = @extended
;~ 		ConsoleWrite('+ _FMIPC_SetSize(' & $iSize & ') : ' & $j & @CRLF)
		If Not $i Then Return SetError($j, $k, 0)
	Else
		$g__FMIPC_Size = $iSize
		Return SetError(0, 0, $iSize)
	EndIf
	$iSize = _FMIPC_Create($g__FMIPC_n)
	Return SetError(@error, 3, $iSize) ; if it fails, ... Need testing.
EndFunc   ;==>_FMIPC_SetSize

; #FUNCTION# ;===============================================================================
;
; Name...........: _FMIPC_SetTimeout
; Description ...: Changes the time it  used in FileMapping
; Syntax.........: _FMIPC_SetTimeout ([$iMSec])
; Parameters ....: $sName - The name for the FileMapping object
; Return values .: Success - Returns timeout in MSecs.
;                          - Sets @error to 0
;                  Failure - Returns pre existing timeout and sets @error:
;                  |1 - The assinged time is not a positive integer
;                  @extended: 1 - when the func. is returning the pre established timeout
; Remarks .......: Not passing a parameter returns the pre established timeout
; Author ........: argumentum
; Modified.......:
; Link ..........;
;
;==========================================================================================
Func _FMIPC_SetTimeout($iMSec = Default)
	If $iMSec = Default Then Return SetError(0, 1, $g__FMIPC_Timeout)
	$iMSec = Int($iMSec)
	If $iMSec < 1 Then Return SetError(1, 0, $g__FMIPC_Timeout)
	$g__FMIPC_Timeout = $iMSec
	Return SetError(0, 0, $g__FMIPC_Timeout)
EndFunc   ;==>_FMIPC_SetTimeout


#Region ### Meaning of error returned ###

Func _FMIPC_Write_ErrStr($i = 0)
	Switch Int($i)
		Case 0
			Return "no error"
		Case 1
			Return "_WinAPI_OpenFileMapping FAILED ( does the objet name exist ? )"
		Case 2
			Return "_WinAPI_MapViewOfFile FAILED"
		Case 3
			Return "Timeout receiving send confirmation.  Hint: _FMIPC_SetTimeout()"
		Case 4
			Return "Timeout waiting for clear to send, never sent.  Hint: _FMIPC_SetTimeout()"
		Case 5
			Return "sending more than declared.  Hint: _FMIPC_SetSize()"
		Case Else
			Return $i & " ???, got me there !!!"
	EndSwitch
EndFunc   ;==>_FMIPC_Write_ErrStr

Func _FMIPC_Create_ErrStr($i = 0)
	Switch Int($i)
		Case 0
			Return "no error"
		Case 1
			Return "The FileMapping object was previously created"
		Case 2
			Return "Unable to create file mapping"
		Case Else
			Return $i & " ???, got me there !!!"
	EndSwitch

EndFunc   ;==>_FMIPC_Create_ErrStr

Func _FMIPC_SetSize_ErrStr($i = 0)
	Switch Int($i)
		Case 0
			Return "no error"
		Case 1
			Return "The FileMapping object was previously created" ; this will change in the near future
		Case 9
			Return "not yet implemented"
		Case Else
			Return $i & " ???, got me there !!!"
	EndSwitch
EndFunc   ;==>_FMIPC_SetSize_ErrStr

#EndRegion ### Meaning of error returned ###
