#include-once ; https://www.autoitscript.com/forum/files/file/500-ondebugmsgbox/
#include <WinApi.au3>
#include <EventLog.au3>
;~ #Au3Stripper_Ignore_Errors_in_Func=__DebugMsgBox_*

; This UDF is thanks to trancexx's code at:
; https://www.autoitscript.com/forum/topic/154081-avoid-autoit-error-message-box-in-unknown-errors/?do=findComment&comment=1111917
; without which these helper functions would not exist.


#cs === User Calltips: ===============================================================================================
_DebugMsgBox_LogFile() ; Set/Get logfile. See remarks for options.
_DebugMsgBox_WinVerStr() ; Set / Get Windows version string
_DebugMsgBox_Au3VerOnScript() Set / Get show of AutoIt version on title when not compiled. Default is True/Show.
_DebugMsgBox_DefaulText() ; Set/Get default text as error header
_DebugMsgBox_NoMsgbox() ; Set/Get hide NoMsgbox state ( True / False )
_DebugMsgBox_NoEventLog() ; Set/Get skip adding EventLog report state ( True / False )
_DebugMsgBox_Extra() ; Set/Get Extra debug string
_DebugMsgBox_FuncParams() ; Set/Get Extra debug string
_DebugMsgBox_ErrReportApp() ; Set/Get script to run in case of crash
_DebugMsgBox_GetUDFversion() ; Get UDF version
_DebugMsgBox_EXEversion([$sVersion]) ; Set / Get executable file version for report
_DebugMsgBox_ScriptVer() ; Get the exe/a3x/au3 version
_DebugMsgBox_AddHookApiError() ; get error return from hook
_DebugMsgBox_PriorLogSize() ; Get / Set log size in kilobytes. Set to zero to disable self maintenance.
_DebugMsgBox_EventLogId() ; Get / Set the event identifier. The event identifier specifies the entry in the message file associated with the event source.
_DebugMsgBox_EventLogCategory() ; Get / Set the event category. This is source specific information the category can have any value.
_DebugMsgBox_EventLogBinaryData() ; Get / Set the array. The number of bytes of event-specific raw (binary) data to write to the log.
#ce === User Calltips: ===============================================================================================

;~ Global Const $VERSION = "1.2.3.4" ; This is an example. Set the "version number" somewhere close to the beginning of your main script (au3/a3x).

#cs === History: =====================================================================================================

What's New in Version 2.2024.1.26
Fixed: ScriptVersion. It now reads from the script the wrapper declaration, else the "Global $VERSION" if found.
changed: "Compiled" is not .exe, just tokenized. It is now set to .exe(1), else is au3/a3x(0).
		 Au3/a3x is functionaly the same hence the change.

What's New in Version 2.2024.1.6
fixed: It'd show FALSE in the title due to an oversight.

What's New in Version 2.2020.6.25
added: _DebugMsgBox_WinVerStr() to set or get the version string.
added: Set/Get show of AutoIt version on title when not compiled. Default is True/Show.
	(this is because I've found myself running beta inadvertently and scratching my head)
changed: _DebugMsgBox_LogFile(). now an empty string will overwrite a log in @TempDir,
	to turn it OFF, set to FALSE. ( read remarks )

What's New in Version 1.2020.5.26
added: logfile self maintenance - _DebugMsgBox_PriorLogSize()
added: error return - _DebugMsgBox_AddHookApiError()
fixed: Au3Stripper error on DllCallbackRegister() and related when /RM is used.
added: TimerDiff() and @UserName to the log file.
added: _DebugMsgBox_EventLogId() - ( no real use for us here, added for completeness )
added: _DebugMsgBox_EventLogCategory() - ( no real use for us here, added for completeness  )
added: _DebugMsgBox_EventLogBinaryData() - ( no real use for us here, added for completeness  )

What's New in Version 1.2020.5.25
added: _DebugMsgBox_GetUDFversion() ; Get UDF version
added: _DebugMsgBox_EXEversion([$sVersion]) ; Set / Get executable file version for report
fixed: needless folder creations

What's New in Version 1.2020.5.24
initial release

#ce === History: =====================================================================================================


#Region Globals

Global $__g__DebugMsgBox_GetUDFversion = "2.2020.6.25"
Global $__g_DebugMsgBox_PriorLogSize = 102400
Global $__g_DebugMsgBox_NoMsgbox = False
Global $__g_DebugMsgBox_NoEventLog = False
Global $__g_DebugMsgBox_Extra = ""
Global $__g_DebugMsgBox_LogFile = @ScriptDir & "\OnDebugMsgBox\" & StringTrimRight(@ScriptName, 4) & '.OnDebugMsgBox.log'
Global $__g_DebugMsgBox_DefaulText = "Please contact the developer."
Global $__g_DebugMsgBox_FuncParams = ""
Global $__g_DebugMsgBox_ErrReportApp = ""
Global $__g_DebugMsgBox_EXEversion = _DebugMsgBox_ScriptVer() ; FileGetVersion(@ScriptFullPath)
Global $__g_DebugMsgBox_EventLogId = 1000
Global $__g_DebugMsgBox_EventLogCategory = 0
Global $__g_DebugMsgBox_EventLogBinaryData[1] = [0]
Global $__g_DebugMsgBox_WinVerStr = ""
Global $__g_DebugMsgBox_AutoItPID = @AutoItPID
Global $__g_DebugMsgBox_AutoItX64 = (@AutoItX64 ? "x64" : "x86")
Global $__g_DebugMsgBox_AutoItVersion = @AutoItVersion
Global $__g_DebugMsgBox_ScriptName = @ScriptName
Global $__g_DebugMsgBox_Compiled = Int(@AutoItExe = @ScriptFullPath) ; @Compiled ; https://www.autoitscript.com/forum/index.php?showtopic=211406
Global $__g_DebugMsgBox_Au3VerOnScript = True
Global $__g_DebugMsgBox_AddHookApiError = 5
Global $__g_DebugMsgBox_TimerInit = TimerInit()

#EndRegion Globals

#Region User functions

; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_WinVerStr
; Description ...: Get / Set the Windows version string for the logfile.
; Syntax ........: _DebugMsgBox_WinVerStr([$vValue = Default])
; Parameters ....: $vValue: can be any string if you don't like the default
; Requirement(s).: None
; Return values .: Windows version string
; Remarks .......: this is used in the logfile only.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_WinVerStr($vValue = Default)
	If $vValue = Default Then
		If Not StringLen($__g_DebugMsgBox_WinVerStr) Then
			$__g_DebugMsgBox_WinVerStr = '"' & FileGetVersion("winver.exe") & '(' & _
					(@OSServicePack <> "" ? @OSServicePack : RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion", "ReleaseId")) & ')" - "' & _
					RegRead("HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion", "ProductName") & '"'
		EndIf
	Else
		$__g_DebugMsgBox_WinVerStr = $vValue
	EndIf
	Return $__g_DebugMsgBox_WinVerStr
EndFunc   ;==>_DebugMsgBox_WinVerStr

; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_EXEversion
; Description ...: Get / Set a LogFile for the OnDebugMsgBox event.
; Syntax ........: _DebugMsgBox_EXEversion([$sVersion = Default])
; Parameters ....:
; Requirement(s).: None
; Return values .: EXE version string
; Remarks .......: this is for the report only. By default is FileGetVersion(@ScriptFullPath)
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_EXEversion($sVersion = Default)
	If $sVersion <> Default Then $__g_DebugMsgBox_EXEversion = $sVersion
	Return $__g_DebugMsgBox_EXEversion
EndFunc   ;==>_DebugMsgBox_EXEversion

; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_ScriptVer
; Description ...: Get / Set a LogFile for the OnDebugMsgBox event.
; Syntax ........: _DebugMsgBox_ScriptVer()
; Parameters ....:
; Requirement(s).: None
; Return values .: attempts to get the exe/a3x/au3 version
; Remarks .......: This is used to initialize _DebugMsgBox_EXEversion()
;					In the case of a3x/au3, a "Global $VERSION =" declaration close to the begining of the code is advised.
; Example........: No
; ===================================================================================================================
ConsoleWrite( _DebugMsgBox_ScriptVer() & @CRLF)
Func _DebugMsgBox_ScriptVer()
	If $__g_DebugMsgBox_EXEversion <> "" Then Return $__g_DebugMsgBox_EXEversion
	If @AutoItExe = @ScriptFullPath Then ; is .exe
		$__g_DebugMsgBox_EXEversion = FileGetVersion(@AutoItExe) ; same as @ScriptFullPath
		Return $__g_DebugMsgBox_EXEversion
	EndIf
	If @Compiled Then ; is .a3x
		If Eval("VE" & "RSION") <> "" And Not @error Then $__g_DebugMsgBox_EXEversion = Eval("VE" & "RSION")
	Else ; is .au3
		Local $n, $aTmp, $aArray = StringSplit(FileRead(@ScriptFullPath, 4096), @CRLF, 1)
		For $n = 1 To $aArray[0]
			If StringInStr($aArray[$n], "#AutoIt3Wrap" & "per_Res_Filev" & "ersion=") = 1 Then
				$aTmp = StringSplit($aArray[$n], "= ;", 0)
				ReDim $aTmp[3]
				If StringInStr($aTmp[1], "#AutoIt3Wrap" & "per_Res_Filev" & "ersion") Then
					$__g_DebugMsgBox_EXEversion = $aTmp[2]
					Return $__g_DebugMsgBox_EXEversion
				EndIf
			EndIf
		Next
		For $n = 1 To $aArray[0]
;~ 			If StringInStr(StringStripWS($aArray[$n], 8), "Gl" & "obal") And StringInStr(StringStripWS($aArray[$n], 8), "$VE" & "RSION=") Then
			If StringInStr(StringStripWS($aArray[$n], 8), "$VE" & "RSION=") Then
				$aTmp = StringSplit(StringStripWS($aArray[$n], 8), "$='"";", 0)
				ReDim $aTmp[5]
				If $aTmp[2] = "VE" & "RSION" Then
					$__g_DebugMsgBox_EXEversion = $aTmp[4]
					Return $__g_DebugMsgBox_EXEversion
				EndIf
			EndIf
		Next
	EndIf
	$__g_DebugMsgBox_EXEversion = "0.0.0.1"
	Return $__g_DebugMsgBox_EXEversion
EndFunc   ;==>_DebugMsgBox_ScriptVer

; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_GetUDFversion
; Description ...: return the UDF version
; Syntax ........: _DebugMsgBox_GetUDFversion()
; Parameters ....:
; Requirement(s).: None
; Return values .: this UDF version
; Remarks .......:
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_GetUDFversion()
	Return $__g__DebugMsgBox_GetUDFversion
EndFunc   ;==>_DebugMsgBox_GetUDFversion

; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_Au3VerOnScript
; Description ...: Get / Set add AutoIt version to title when NOT compiled.
; Syntax ........: _DebugMsgBox_Au3VerOnScript([$bValue = Default])
; Parameters ....: Default, True, False
; Requirement(s).: None
; Return values .: True, False
; Remarks .......: Compiled is as EXE or A3X, not compiled is AU3 text script only.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_Au3VerOnScript($bValue = Default)
	If Not (IsKeyword($bValue) Or $bValue == "") Then $__g_DebugMsgBox_Au3VerOnScript = ($bValue ? True : False)
	Return $__g_DebugMsgBox_Au3VerOnScript
EndFunc   ;==>_DebugMsgBox_Au3VerOnScript

; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_LogFile
; Description ...: Get / Set a LogFile for the OnDebugMsgBox event.
; Syntax ........: _DebugMsgBox_LogFile([$sLogFile = Default])
; Parameters ....: logfile name
; Requirement(s).: None
; Return values .: logfile name
; Remarks .......: if you don't declare a file or a path, it will create a sub folder named "OnDebugMsgBox",
;					if is a path, will create the log file there, and if is a filename it will use the filename.
;					If set to "" file writting will be done as overwrite in @TempDir, keeping only that last error.
;					To purposefully not have a file written, set to FALSE.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_LogFile($sLogFile = Default)
	If $sLogFile == False Then
		$__g_DebugMsgBox_LogFile = False
	ElseIf $sLogFile = Default Then
		Return $__g_DebugMsgBox_LogFile
	ElseIf $sLogFile == "" Then
		$__g_DebugMsgBox_LogFile = ""
	Else
		If StringInStr(FileGetAttrib($sLogFile), "D") Or StringRight($sLogFile, 1) = "\" Then
			If StringRight($sLogFile, 1) <> "\" Then $sLogFile &= "\"
			$__g_DebugMsgBox_LogFile = $sLogFile & StringTrimRight(@ScriptName, 4) & '.OnDebugMsgBox.log'
		Else
			$__g_DebugMsgBox_LogFile = $sLogFile
		EndIf
	EndIf
	Return $__g_DebugMsgBox_LogFile
EndFunc   ;==>_DebugMsgBox_LogFile


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_DefaulText
; Description ...: Get / Set a header text for the MsgBox
; Syntax ........: _DebugMsgBox_DefaulText([$sHeader = Default])
; Parameters ....: see remarks
; Requirement(s).: None
; Return values .: Header text
; Remarks .......: the default is "Please contact the developer." but you can change that with this,
;					calling without a string will return the current header text.
;					To remove this header, call with "" as the text string.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_DefaulText($sHeader = Default)
	If $sHeader <> Default Then $__g_DebugMsgBox_DefaulText = $sHeader
	Return $__g_DebugMsgBox_DefaulText
EndFunc   ;==>_DebugMsgBox_DefaulText


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_NoEventLog
; Description ...: Get / Set EvenLog reporting
; Syntax ........: _DebugMsgBox_NoEventLog([$iBool = Default])
; Parameters ....: $iBool = true / false / default
; Requirement(s).: None
; Return values .: state: True or False
; Remarks .......: the default is False, to skip adding an EvenLog report set to True,
;					calling without a value will return the current state.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_NoEventLog($iBool = Default)
	If $iBool <> Default And $iBool <> "" Then $__g_DebugMsgBox_NoEventLog = Int($iBool) > 0
	Return $__g_DebugMsgBox_NoEventLog
EndFunc   ;==>_DebugMsgBox_NoEventLog


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_NoMsgbox
; Description ...: Get / Set to hide the debug MsgBox
; Syntax ........: _DebugMsgBox_NoMsgbox([$iBool = Default])
; Parameters ....: $iBool = true / false / default
; Requirement(s).: None
; Return values .: state: True of False
; Remarks .......: the default is False, to skip the MsgBox set to True,
;					calling without a value will return the current state.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_NoMsgbox($iBool = Default)
	If $iBool <> Default And $iBool <> "" Then $__g_DebugMsgBox_NoMsgbox = Int($iBool) > 0
	Return $__g_DebugMsgBox_NoMsgbox
EndFunc   ;==>_DebugMsgBox_NoMsgbox


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_Extra
; Description ...: Get / Set a value to use in the report
; Syntax ........: _DebugMsgBox_Extra([$iBool = Default])
; Parameters ....:
; Requirement(s).: None
; Return values .: String
; Remarks .......: Without parameters it will return the value held.
;					calling with a value will set it in case of a crash
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_Extra($sStr = Default)
	If $sStr <> Default Then $__g_DebugMsgBox_Extra = $sStr
	Return $__g_DebugMsgBox_Extra
EndFunc   ;==>_DebugMsgBox_Extra


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_FuncParams
; Description ...: Get / Set a value to use in the report
; Syntax ........: _DebugMsgBox_Extra([$iBool = Default])
; Parameters ....:
; Requirement(s).: None
; Return values .: String
; Remarks .......: Without parameters it will return the value held.
;					calling with a value will set it in case of a crash
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_FuncParams($sStr = Default)
	If $sStr <> Default Then $__g_DebugMsgBox_FuncParams = $sStr
	Return $__g_DebugMsgBox_FuncParams
EndFunc   ;==>_DebugMsgBox_FuncParams


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_ErrReportApp
; Description ...: Get / Set an App. to run and handle the error event
; Syntax ........: _DebugMsgBox_ErrReportApp([$sFile = Default])
; Parameters ....: see Remarks
; Requirement(s).: None
; Return values .: String
; Remarks .......: Without parameters it will return the value held.
;					calling with a value will set it in case of a crash.
;					The parameters passed to the app. are: '/OnDebugMsgBox "' & _DebugMsgBox_LogFile() & '" "' & $timestamp & '"'
;					Then the app. would read the log and do, what you code it to do.
; Example........: Yes, run to clearly understand it's workings
; ===================================================================================================================
Func _DebugMsgBox_ErrReportApp($sFile = Default)
	If $sFile = Default Then
		Return $__g_DebugMsgBox_ErrReportApp
	ElseIf $sFile = "" Then
		$__g_DebugMsgBox_ErrReportApp = $sFile
		Return $sFile
	Else
;~ 		If Int(StringReplace(StringReplace(FileGetVersion($sFile), ".", ""), ",", "")) Or FileGetSize($sFile) Then
		If FileGetSize($sFile) Then
			$__g_DebugMsgBox_ErrReportApp = $sFile
			Return $sFile
		EndIf
	EndIf
	Return SetError(1, 0, $__g_DebugMsgBox_ErrReportApp)
EndFunc   ;==>_DebugMsgBox_ErrReportApp


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_AddHookApiError
; Description ...: Get error code from the main function. 0 = success
; Syntax ........: _DebugMsgBox_AddHookApiError()
; Parameters ....: None
; Requirement(s).: None
; Return values .: integer
; Remarks .......: The window hook is UDF loaded. This func. will return if there as an error hooking.
;					The function should return zero(0) if running OK
;				  Error 0 = OK
;				  Error 1,2,3,4 = failed
;				  Error 5 = the function never got called
; Example........: Yes, run to clearly understand it's workings
; ===================================================================================================================
Func _DebugMsgBox_AddHookApiError()
	Return $__g_DebugMsgBox_AddHookApiError
EndFunc   ;==>_DebugMsgBox_AddHookApiError


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_PriorLogSize
; Description ...: Get / Set max. log file size
; Syntax ........: _DebugMsgBox_PriorLogSize([$iSizeInKiloBytes = Default])
; Parameters ....: $iSizeInKiloBytes or Default to get the size in kilobytes
; Requirement(s).: None
; Return values .: integer
; Remarks .......: Is not uncommon to find logs that are so big and old that are practically useless.
;                  By default when a log is 100k is renamed to "prior" and a prior to that deleted,
;                   that way you have up to 2 manageable sized, self maintained logs to read.
;				   Set to zero to skip this feature ( and have a forever single log file )
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_PriorLogSize($iSizeInKiloBytes = Default)
	If IsInt($iSizeInKiloBytes) Then $__g_DebugMsgBox_PriorLogSize = $iSizeInKiloBytes * 1024
	Return $__g_DebugMsgBox_PriorLogSize / 1024
EndFunc   ;==>_DebugMsgBox_PriorLogSize


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_EventLogId
; Description ...: Get / Set Event ID
; Syntax ........: _DebugMsgBox_EventLogId([$iEventID = Default])
; Parameters ....:
; Requirement(s).: None
; Return values .: integer
; Remarks .......: for more info read https://docs.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-reporteventw
;                  The default is 1000. We have no use for it but, you can change it with this func.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_EventLogId($iEventID = Default)
	If IsInt($iEventID) Then $__g_DebugMsgBox_EventLogId = $iEventID
	Return $__g_DebugMsgBox_EventLogId
EndFunc   ;==>_DebugMsgBox_EventLogId


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_EventLogCategory
; Description ...: Get / Set Event ID
; Syntax ........: _DebugMsgBox_EventLogCategory([$iCategory = Default])
; Parameters ....:
; Requirement(s).: None
; Return values .: integer
; Remarks .......: for more info read https://docs.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-reporteventw
;                  The default is 0 ( None ). We have no use for it but, you can change it with this func.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_EventLogCategory($iCategory = Default)
	If IsInt($iCategory) Then $__g_DebugMsgBox_EventLogCategory = $iCategory
	Return $__g_DebugMsgBox_EventLogCategory
EndFunc   ;==>_DebugMsgBox_EventLogCategory


; #FUNCTION# =========================================================================================================
; Name...........: _DebugMsgBox_EventLogBinaryData
; Description ...: Get / Set Event ID
; Syntax ........: _DebugMsgBox_EventLogBinaryData([$aData = Default])
; Parameters ....:
; Requirement(s).: None
; Return values .: integer
; Remarks .......: for more info read:
;						https://www.autoitscript.com/autoit3/docs/libfunctions/_EventLog__Report.htm
;						https://docs.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-reporteventw
;                  The default is [0] ( empty ). We have no use for it but, you can change it with this func.
; Example........: No
; ===================================================================================================================
Func _DebugMsgBox_EventLogBinaryData($aData = Default)
	If IsArray($aData) Then $__g_DebugMsgBox_EventLogBinaryData = $aData
	Return $__g_DebugMsgBox_EventLogBinaryData
EndFunc   ;==>_DebugMsgBox_EventLogBinaryData

#EndRegion User functions


#Region internal functions

_DebugMsgBox_WinVerStr()
__DebugMsgBox_AddHookApi()
$__g_DebugMsgBox_AddHookApiError = @error

Func __DebugMsgBox_Intercept_MessageBoxW($hWnd, $sText, $sTitle, $iType) ; internal
	If StringInStr($sTitle, 'AutoIt') = 1 Then ; Or StringInStr($sTitle, 'Error:') Then
		AutoItWinSetTitle(StringTrimRight(@ScriptName, 4) & ' code error =(')
		Local $iFileOpenFlag = 9, $ts = @YEAR & '.' & @MON & '.' & @MDAY & ' ' & @HOUR & ':' & @MIN & ':' & @SEC & '.' & @MSEC
		$sTitle = StringReplace($sTitle, "AutoIt", StringTrimRight(@ScriptName, 4))
		If StringLen($__g_DebugMsgBox_DefaulText) Then $__g_DebugMsgBox_DefaulText &= @CRLF & @CRLF & '@'
		$sText = $__g_DebugMsgBox_DefaulText & $sText
		Local $sTextRep = StringReplace($sText, @CR, @LF)
		$sTextRep = StringReplace($sTextRep, @LF & @LF, @LF)
		$sTextRep = StringReplace($sTextRep, @LF & @LF, @LF)
		$sTextRep = StringReplace($sTextRep, @LF, @CRLF)

		If $__g_DebugMsgBox_PriorLogSize And StringLen($__g_DebugMsgBox_LogFile) Then
			Local $sPriorLog = StringTrimRight($__g_DebugMsgBox_LogFile, 4) & ".prior.log"
			If FileGetSize($__g_DebugMsgBox_LogFile) > $__g_DebugMsgBox_PriorLogSize Then
				If FileExists($sPriorLog) Then FileDelete($sPriorLog)
				FileMove($__g_DebugMsgBox_LogFile, $sPriorLog)
			EndIf
		EndIf

		If StringLen($__g_DebugMsgBox_LogFile) = 0 Then
			$__g_DebugMsgBox_LogFile = @TempDir & "\" & StringTrimRight(@ScriptName, 4) & '.OnDebugMsgBox.log'
			$iFileOpenFlag = 10
		EndIf

		If Not $__g_DebugMsgBox_LogFile == False Then
			Local $nLine = @TAB & @CRLF, $hFile = FileOpen($__g_DebugMsgBox_LogFile, $iFileOpenFlag)
			If $hFile = -1 Then ; ..just in case the path could not be created, have a log somewhere.
				$__g_DebugMsgBox_LogFile = @TempDir & "\" & StringTrimRight(@ScriptName, 4) & '.OnDebugMsgBox.log'
				$hFile = FileOpen($__g_DebugMsgBox_LogFile, $iFileOpenFlag)
			EndIf
			If $hFile <> -1 Then
				FileWrite($hFile, $nLine & '--- start of OnDebugMsgBox report ---' & $nLine & $ts & $nLine & _
						'+ Self: "' & @ScriptFullPath & '" [v' & $__g_DebugMsgBox_EXEversion & '][v' & $__g_DebugMsgBox_AutoItVersion & _
						' (' & $__g_DebugMsgBox_AutoItX64 & ')][PID: ' & $__g_DebugMsgBox_AutoItPID & '][UserName: ' & @UserName & _
						'][IsAdmin: ' & IsAdmin() & '][TimerDiff: ' & TimerDiff($__g_DebugMsgBox_TimerInit) & ']' & $nLine & _
						'+ WinVer: ' & $__g_DebugMsgBox_WinVerStr & $nLine & _
						'+ CmdLineRaw: ' & $CmdLineRaw & $nLine & _
						'+ DebugFuncParams: ' & StringReplace($__g_DebugMsgBox_FuncParams & $nLine, $nLine & $nLine, $nLine) & _
						'+ DebugExtra: ' & StringReplace($__g_DebugMsgBox_Extra & $nLine, $nLine & $nLine, $nLine) & _
						'+ Text: ' & $sTextRep & $nLine & _
						'--- end of OnDebugMsgBox report ---' & $nLine)
				FileClose($hFile)
			EndIf
		EndIf
		If Not $__g_DebugMsgBox_NoEventLog Then __DebugMsgBox_EventLog_Report($sTextRep)
		If StringLen($__g_DebugMsgBox_ErrReportApp) Then ShellExecute($__g_DebugMsgBox_ErrReportApp, '/OnDebugMsgBox "' & $__g_DebugMsgBox_LogFile & '" "' & $ts & '"')
		If $__g_DebugMsgBox_NoMsgbox Then Return ; just return to not see the msgbox ; https://www.autoitscript.com/forum/topic/143250-grab-runtime-errors-when-script-is-run-from-a-service/?do=findComment&comment=1009391
		$sTitle &= (($__g_DebugMsgBox_Compiled = 0 And $__g_DebugMsgBox_Au3VerOnScript) ? " (v" & $__g_DebugMsgBox_AutoItVersion & ")(" & $__g_DebugMsgBox_AutoItX64 & ")" : "")
	EndIf
	Local $aCall = DllCall("user32.dll", "int", "MessageBoxW", "hwnd", $hWnd, "wstr", $sText, "wstr", $sTitle, "uint", $iType)
	If @error Or Not $aCall[0] Then Return 0
	Return $aCall[0]
EndFunc   ;==>__DebugMsgBox_Intercept_MessageBoxW

Func __DebugMsgBox_EventLog_Report($sText)
	Local Const $EVENTLOG_ERROR_TYPE = 0x0001 ; Error event
	Local $hEventLog ; https://docs.microsoft.com/en-us/windows/win32/api/winbase/nf-winbase-reporteventw
	$hEventLog = _EventLog__Open("", $__g_DebugMsgBox_ScriptName)
	_EventLog__Report($hEventLog, $EVENTLOG_ERROR_TYPE, $__g_DebugMsgBox_EventLogCategory, $__g_DebugMsgBox_EventLogId, @UserName, $sText, $__g_DebugMsgBox_EventLogBinaryData)
	_EventLog__Close($hEventLog)
EndFunc   ;==>__DebugMsgBox_EventLog_Report

Func __DebugMsgBox_AddHookApi_WinAPI_GetModuleHandle($sModuleName) ; internal
	Local $sModuleNameType = "wstr"
	If $sModuleName = "" Then
		$sModuleName = 0
		$sModuleNameType = "ptr"
	EndIf
	Local $aResult = DllCall("kernel32.dll", "handle", "GetModuleHandleW", $sModuleNameType, $sModuleName)
	If @error Then Return SetError(@error, @extended, 0)
	Return $aResult[0]
EndFunc   ;==>__DebugMsgBox_AddHookApi_WinAPI_GetModuleHandle

Func __DebugMsgBox_AddHookApi_WinAPI_StringLenA(Const ByRef $tString) ; internal
	Local $aResult = DllCall("kernel32.dll", "int", "lstrlenA", "struct*", $tString)
	If @error Then Return SetError(@error, @extended, 0)
	Return $aResult[0]
EndFunc   ;==>__DebugMsgBox_AddHookApi_WinAPI_StringLenA

Func __DebugMsgBox_AddHookApi() ;$sModuleName, $vFunctionName, $vNewFunction, $sRet = "", $sParams = "") ; internal
;~ 	Local $sModuleName = "user32.dll",
;~ 	Local $vFunctionName = "MessageBoxW",
;~ 	Local $vNewFunction = FuncName(__DebugMsgBox_Intercept_MessageBoxW),
;~ 	Local $sRet = "int",
;~ 	Local $sParams = "hwnd;wstr;wstr;uint"

	Local Static $pImportDirectory, $hInstance
	Local Const $IMAGE_DIRECTORY_ENTRY_IMPORT = 1
	If Not $pImportDirectory Then
		$hInstance = __DebugMsgBox_AddHookApi_WinAPI_GetModuleHandle(0)
		$pImportDirectory = __DebugMsgBox_ImageDirectoryEntryToData($hInstance, $IMAGE_DIRECTORY_ENTRY_IMPORT)
		If @error Then Return SetError(1, 0, 0)
	EndIf
;~ 	Local $iIsInt = IsInt($vFunctionName)
;~ 	Local $iRestore = Not IsString($vNewFunction)
	Local $tIMAGE_IMPORT_MODULE_DIRECTORY
	Local $pDirectoryOffset = $pImportDirectory
	Local $tModuleName
	Local $iInitialOffset, $iInitialOffset2
	Local $iOffset2
	Local $tBufferOffset2, $iBufferOffset2
	Local $tBuffer, $tFunctionOffset, $pOld, $fMatch, $pModuleName, $pFuncName
	Local Const $PAGE_READWRITE = 0x04
	While 1
		$tIMAGE_IMPORT_MODULE_DIRECTORY = DllStructCreate("dword RVAOriginalFirstThunk;" & _
				"dword TimeDateStamp;" & _
				"dword ForwarderChain;" & _
				"dword RVAModuleName;" & _
				"dword RVAFirstThunk", _
				$pDirectoryOffset)
		If Not DllStructGetData($tIMAGE_IMPORT_MODULE_DIRECTORY, "RVAFirstThunk") Then ExitLoop
		$pModuleName = $hInstance + DllStructGetData($tIMAGE_IMPORT_MODULE_DIRECTORY, "RVAModuleName")
		$tModuleName = DllStructCreate("char Name[" & __DebugMsgBox_AddHookApi_WinAPI_StringLenA($pModuleName) & "]", $pModuleName)
		If DllStructGetData($tModuleName, "Name") = "user32.dll" Then ; function from this module
			$iInitialOffset = $hInstance + DllStructGetData($tIMAGE_IMPORT_MODULE_DIRECTORY, "RVAFirstThunk")
			$iInitialOffset2 = $hInstance + DllStructGetData($tIMAGE_IMPORT_MODULE_DIRECTORY, "RVAOriginalFirstThunk")
			If $iInitialOffset2 = $hInstance Then $iInitialOffset2 = $iInitialOffset
			$iOffset2 = 0
			While 1
				$tBufferOffset2 = DllStructCreate("dword_ptr", $iInitialOffset2 + $iOffset2)
				$iBufferOffset2 = DllStructGetData($tBufferOffset2, 1)
				If Not $iBufferOffset2 Then ExitLoop
;~ 				If $iIsInt Then
;~ 					If BitAND($iBufferOffset2, 0xFFFFFF) = "MessageBoxW" Then $fMatch = True ; wanted function
;~ 				Else
				$pFuncName = $hInstance + $iBufferOffset2 + 2     ; 2 is size od "word", see line below...
				$tBuffer = DllStructCreate("word Ordinal; char Name[" & __DebugMsgBox_AddHookApi_WinAPI_StringLenA($pFuncName) & "]", $hInstance + $iBufferOffset2)
				If DllStructGetData($tBuffer, "Name") == "MessageBoxW" Then $fMatch = True     ; wanted function
;~ 				EndIf
				If $fMatch Then
					$tFunctionOffset = DllStructCreate("ptr", $iInitialOffset + $iOffset2)
					__DebugMsgBox_VirtualProtect(DllStructGetPtr($tFunctionOffset), DllStructGetSize($tFunctionOffset), $PAGE_READWRITE)
					If @error Then Return SetError(3, 0, 0)
					$pOld = DllStructGetData($tFunctionOffset, 1)
;~ 					If $iRestore Then
;~ 						DllStructSetData($tFunctionOffset, 1, "__DebugMsgBox_Intercept_MessageBoxW");$vNewFunction)
;~ 					Else
					DllStructSetData($tFunctionOffset, 1, DllCallbackGetPtr(DllCallbackRegister("__DebugMsgBox_Intercept_MessageBoxW", "int", "hwnd;wstr;wstr;uint")))
;~ 					EndIf
					Return $pOld
				EndIf
				$iOffset2 += DllStructGetSize($tBufferOffset2)
			WEnd
			ExitLoop
		EndIf
		$pDirectoryOffset += 20 ; size of $tIMAGE_IMPORT_MODULE_DIRECTORY
	WEnd
	Return SetError(4, 0, 0)
EndFunc   ;==>__DebugMsgBox_AddHookApi

Func __DebugMsgBox_VirtualProtect($pAddress, $iSize, $iProtection) ; internal
	Local $aCall = DllCall("kernel32.dll", "bool", "VirtualProtect", "ptr", $pAddress, "dword_ptr", $iSize, "dword", $iProtection, "dword*", 0)
	If @error Or Not $aCall[0] Then Return SetError(1, 0, 0)
	Return 1
EndFunc   ;==>__DebugMsgBox_VirtualProtect

Func __DebugMsgBox_ImageDirectoryEntryToData($hInstance, $iDirectoryEntry) ; internal
	; Get pointer to data
	Local $pPointer = $hInstance
	; Start processing passed binary data. 'Reading' PE format follows.
	Local $tIMAGE_DOS_HEADER = DllStructCreate("char Magic[2];" & _
			"word BytesOnLastPage;" & _
			"word Pages;" & _
			"word Relocations;" & _
			"word SizeofHeader;" & _
			"word MinimumExtra;" & _
			"word MaximumExtra;" & _
			"word SS;" & _
			"word SP;" & _
			"word Checksum;" & _
			"word IP;" & _
			"word CS;" & _
			"word Relocation;" & _
			"word Overlay;" & _
			"char Reserved[8];" & _
			"word OEMIdentifier;" & _
			"word OEMInformation;" & _
			"char Reserved2[20];" & _
			"dword AddressOfNewExeHeader", _
			$pPointer)
	Local $sMagic = DllStructGetData($tIMAGE_DOS_HEADER, "Magic")
	; Check if it's valid format
	If Not ($sMagic == "MZ") Then Return SetError(1, 0, 0) ; MS-DOS header missing. Btw 'MZ' are the initials of Mark Zbikowski in case you didn't know.
	; Move pointer
	$pPointer += DllStructGetData($tIMAGE_DOS_HEADER, "AddressOfNewExeHeader") ; move to PE file header
	; In place of IMAGE_NT_SIGNATURE structure
	Local $tIMAGE_NT_SIGNATURE = DllStructCreate("dword Signature", $pPointer)
	; Check signature
	If DllStructGetData($tIMAGE_NT_SIGNATURE, "Signature") <> 17744 Then ; IMAGE_NT_SIGNATURE
		Return SetError(2, 0, 0) ; wrong signature. For PE image should be "PE\0\0" or 17744 dword.
	EndIf
	; Move pointer
	$pPointer += 4 ; size of $tIMAGE_NT_SIGNATURE structure
	; In place of IMAGE_FILE_HEADER structure
	; Move pointer
	$pPointer += 20 ; size of $tIMAGE_FILE_HEADER structure
	; Determine the type
	Local $tMagic = DllStructCreate("word Magic;", $pPointer)
	Local $iMagic = DllStructGetData($tMagic, 1)
;~ 	Local $tIMAGE_OPTIONAL_HEADER
	If $iMagic = 267 Then ; x86 version
		; Move pointer
		$pPointer += 96 ; size of $tIMAGE_OPTIONAL_HEADER
	ElseIf $iMagic = 523 Then ; x64 version
		; Move pointer
		$pPointer += 112 ; size of $tIMAGE_OPTIONAL_HEADER
	Else
		Return SetError(3, 0, 0) ; unsupported module type
	EndIf
	; Validate input by checking available number of structures that are in the module
	Local Const $IMAGE_NUMBEROF_DIRECTORY_ENTRIES = 16 ; predefined value that PE modules always use (AutoIt certainly)
	If $iDirectoryEntry > $IMAGE_NUMBEROF_DIRECTORY_ENTRIES - 1 Then Return SetError(4, 0, 0) ; invalid input
	; Calculate the offset to wanted entry (every entry is 8 bytes)
	$pPointer += 8 * $iDirectoryEntry
	; At place of correst directory entry
	Local $tIMAGE_DIRECTORY_ENTRY = DllStructCreate("dword VirtualAddress; dword Size", $pPointer)
	; Collect data
	Local $pAddress = DllStructGetData($tIMAGE_DIRECTORY_ENTRY, "VirtualAddress")
	If $pAddress = 0 Then Return SetError(5, 0, 0) ; invalid input
	; $pAddress is RVA, add it to base address
	Return $hInstance + $pAddress
EndFunc   ;==>__DebugMsgBox_ImageDirectoryEntryToData

#EndRegion internal functions

